#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>


/*
*    Converts URL to /path/file
*    Converts %XX to byte value, E.G.: '%41' to 'A'.
*
*    Copyright (c) 2020, 2021 R.J. van der Putten, Leiden, Holland, 
*    rob at sput dot nl.
* 
*    This program is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License version 3 as
*    published by the Free Software Foundation.
* 
*    This program is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*/


#define DOCROOT "/var/www"
#define INDEX   "index.html"
#define PUBHTML "WWW"

void hlp(void)
{
    fprintf(stderr, \
      "Usage: url2file [-d Doc_Root] [-i] [-p Public_HTML_Dir] [-s]\n");
    fprintf(stderr, "-i: Add index.html to directories\n");
    fprintf(stderr, "-s: Allow spaces in filenames\n");
    fprintf(stderr, \
      "Copyright (c) 2020, 2021 R.J. van der Putten, Leiden, Holland, ");
    fprintf(stderr, "rob at sput dot nl\n");
    fprintf(stderr, \
      "GNU GPL applies. See gnu.org for more information.\n");
}


int main(int argc, char **argv)
{
    int i, indflg, j, len, opt, ret, spcflg;
    unsigned int u;
    char ch, docroot[2048], line[4096], line2[4096] , pubhtml[2048];

    ch = 0;
    i = 0;
    indflg = 0;
    j = 0;
    len = 0;
    opt = 0;
    ret = 0;
    spcflg = 0;
    u = 0;
    memset(docroot, 0, 2048);
    memset(line,    0, 4096);
    memset(line2,   0, 4096);
    memset(pubhtml, 0, 2048);

    /*  Defaults  */
    strcpy(docroot, DOCROOT);
    strcpy(pubhtml, PUBHTML);

    while ((opt = getopt(argc, argv, "d:hip:s")) != -1) {
	switch (opt) {
	    case 'd':
		strncpy(docroot, optarg, 2047);
		break;
	    case 'h':
		hlp();
		exit(0);
	    case 'i':
		indflg = 1;
		break;
	    case 'p':
		strncpy(pubhtml, optarg, 2047);
		break;
	    case 's':
		spcflg = 1;
		break;
	    default:
		hlp();
		exit(1);
	}
    }

    while (fgets(line, 4088, stdin)) {
	j = 0;
	len = strlen(line) - 1;
	if (line[len] == 10) {
	    line[len] = 0;
	    len--;
	}
	if(line[0] == 0)
	    continue;
	if (line[len] == 13) {
	    line[len] = 0;
	    len--;
	}
	if(line[0] == 0)
	    continue;
	len++;

	i = 0;
	j = 0;
        while (i < len) {
	    /*  De-escape  */
	    if (line[i] == '%' && isxdigit(line[i + 1]) != 0 && \
	      isxdigit(line[i + 2]) != 0) {
		ret = sscanf(line + i + 1, "%2x", &u);
		if (ret == 1) {
		    if (u == 0 || (spcflg == 0 && u == 32)) {
			fprintf(stderr, "Illegal filename: %s\n", line);
			exit(1);
		    }
		    line2[j] = (char) (0xFF & u);
                    i += 2;
		} else {
		    line2[j] = line[i];
		}
	    } else {
		line2[j] = line[i];
	    }
	    i++;
	    j++;
	    line2[j] = 0;    /*  Terminate  */
	}
	len = j;

	i = 0;
	j = 0;
	if (strncmp(line2, "/~", 2) == 0) {
	    /*  Pub HTML  */
	    i = 2;
	    while (line2[i] != '/' && i < len) {
		i++;
	    }
	    ch = line2[i];
	    line2[i] = 0;
	    printf("/home/%s/%s", line2 + 2, PUBHTML);
	    line2[i] = ch;
	    j = i;
	} else {
	    /*  Doc root  */
	    printf("%s", docroot);
	}

	printf("%s", line2 + j);
	if(indflg != 0 && line2[len - 1] == '/')
	    printf("%s", INDEX);
	printf("\n");
    }

    return(0);
}

